var ec2_api_list = {
    'us-east-1' : "2008-12-01",
    'eu-west-1' : "2008-12-01",
};

var ec2_httpclient = {
    handler : null,
    uri     : null,
    auxObj  : null,
    serviceURL : null,
    accessCode : null,
    secretKey : null,
    timers : {},

    USER_AGENT : "Elasticfox/1.6-000090",

    API_VERSION : "2008-12-01",

    // TESTING ONLY
    kpcreated : false,
    imgregistered : false,
    launched : false,
    terminated : false,

    getNsResolver : function() {
        var client = this;
        return function(prefix) {
            var ns = {
            's':  "http://schemas.xmlsoap.org/soap/envelope/",  // SOAP namespace
            'ec2': "http://ec2.amazonaws.com/doc/"+client.API_VERSION+"/"   // EC2 namespace, must match request version
            };
            return ns[prefix] || null;
        }
    },

    setCredentials : function (accessCode, secretKey) {
        this.accessCode = accessCode;
        this.secretKey = secretKey;
    },

    setEndpoint : function (endpoint) {
        if (endpoint != null) {
            this.serviceURL = endpoint.url;
        }
    },

    newInstance : function() {
        var xmlhttp;
        if (!xmlhttp && typeof XMLHttpRequest != 'undefined') {
            try {
                xmlhttp = new XMLHttpRequest();
            } catch (e) {
                xmlhttp = false
            }
        }
        return xmlhttp;
    },

    sigParamCmp : function(x, y) {
        if (x[0].toLowerCase() < y[0].toLowerCase ()) {
            return -1;
        }
        if (x[0].toLowerCase() > y[0].toLowerCase()) {
           return 1;
        }
        return 0;
    },

    addZero : function(vNumber) {
        return ((vNumber < 10) ? "0" : "") + vNumber;
    },

    formatDate : function(vDate, vFormat) {
        var vDay              = this.addZero(vDate.getUTCDate());
        var vMonth            = this.addZero(vDate.getUTCMonth()+1);
        var vYearLong         = this.addZero(vDate.getUTCFullYear());
        var vYearShort        = this.addZero(vDate.getUTCFullYear().toString().substring(3,4));
        var vYear             = (vFormat.indexOf("yyyy")>-1?vYearLong:vYearShort);
        var vHour             = this.addZero(vDate.getUTCHours());
        var vMinute           = this.addZero(vDate.getUTCMinutes());
        var vSecond           = this.addZero(vDate.getUTCSeconds());
        var vDateString       = vFormat.replace(/dd/g, vDay).replace(/MM/g, vMonth).replace(/y{1,4}/g, vYear);
        vDateString           = vDateString.replace(/hh/g, vHour).replace(/mm/g, vMinute).replace(/ss/g, vSecond);
        return vDateString;
    },

    queryEC2 : function (action, params, objActions, isSync, reqType, callback) {
        if (this.accessCode == null || this.accessCode == "") {
            log ("No Access Code for user");
            return;
        }

        if (this.serviceURL == null || this.serviceURL == "") {
            this.setEndpoint(ec2ui_session.getActiveEndpoint());
        }

        while(true) {
            try {
                var rsp = this.queryEC2Impl(action, params, objActions, isSync, reqType, callback);
                if (rsp.hasErrors) {
                    if (!this.errorDialog(
                        "EC2 responded with an error for "+action,
                        rsp.faultCode,
                        rsp.requestId,
                        rsp.faultString)) {
                        return false;
                    }
                } else {
                    objActions.onResponseComplete(rsp);
                    return true;
                }
            } catch (e) {
                alert("An error occurred while calling "+action+"\n"+e);
                return null;
            }
        }
    },

    errorDialog : function(msg, code, rId, fStr) {
        var retry = {value:null};
        window.openDialog(
            "chrome://ec2ui/content/dialog_retry_cancel.xul",
            null,
            "chrome,centerscreen,modal",
            msg,
            code,
            rId,
            fStr,
            retry);
        return retry.value;
    },

    queryEC2Impl : function (action, params, objActions, isSync, reqType, callback) {
        var curTime = new Date();
        var formattedTime = this.formatDate(curTime, "yyyy-MM-ddThh:mm:ssZ");

        var sigValues = new Array();
        sigValues.push(new Array("Action", action));
        sigValues.push(new Array("AWSAccessKeyId", this.accessCode));
        sigValues.push(new Array("SignatureVersion","1"));
        sigValues.push(new Array("Version",this.API_VERSION));
        sigValues.push(new Array("Timestamp",formattedTime));

        // Mix in the additional parameters. params must be an Array of tuples as for sigValues above
        for (var i = 0; i < params.length; i++) {
            sigValues.push(params[i]);
        }

        // Sort the parameters by their lowercase name
        sigValues.sort(this.sigParamCmp);

        // Construct the string to sign and query string
        var strSig = "";
        var queryParams = "?";
        for (var i = 0; i < sigValues.length; i++) {
            strSig += sigValues[i][0] + sigValues[i][1];
            queryParams += sigValues[i][0] + "=" + encodeURIComponent(sigValues[i][1]);
            if (i < sigValues.length-1)
                queryParams += "&";
        }

        log("StrSig ["+strSig+"]");
        log("Params ["+queryParams+"]");

        var sig = b64_hmac_sha1(this.secretKey, strSig);
        log("Sig ["+sig+"]");

        //
        // In order to support more endpoints, we now store the endpoint
        // URL in the client instead of getting it from the prefs object
        // var url = ec2ui_prefs.getServiceURL() + "/" + queryParams + "&Signature="+encodeURIComponent(sig)
        //
        var url = this.serviceURL + "/" + queryParams + "&Signature="+encodeURIComponent(sig)
        log("URL ["+url+"]");

        var timerKey = strSig+":"+formattedTime;

        if (!ec2ui_prefs.isOfflineEnabled()) {
            var xmlhttp = this.newInstance();
            xmlhttp.open("GET", url, !isSync);
            xmlhttp.setRequestHeader("User-Agent", this.USER_AGENT);
            this.startTimer(timerKey, ec2ui_prefs.getRequestTimeout(), xmlhttp.abort);
            try {
                xmlhttp.send(null);
            } catch(e) {
                if (!this.stopTimer(timerKey)) {
                    // A timer didn't exist, this is unexpected
                    throw e;
                }

                var responseObject = {
                    callback: callback,
                    requestType : reqType,
                    faultCode : "Request Error",
                    faultString : "Please check your EC2 URL for correctness. Alternatively, delete the value in ec2ui.endpoints using about:config and retry.",
                    requestId : "",
                    hasErrors : true
                };
                return responseObject;
            }
        }

        // FIXME: Clean this lot up. Get rid of the async stuff if we're not using it.
        if (isSync) {
            log("Sync Response = " + xmlhttp.status + "("+xmlhttp.readyState+"): " + xmlhttp.responseText);

            if (xmlhttp.status >= 200 && xmlhttp.status < 300 ) {
                this.stopTimer(timerKey);
                var responseObject = {
                    xmlDoc: xmlhttp.responseXML,
                    strHeaders: xmlhttp.getAllResponseHeaders(),
                    callback: callback,
                    requestType : reqType
                };

                return responseObject;
            } else {
                this.stopTimer(timerKey);
                return this.unpackXMLErrorRsp(xmlhttp, reqType, callback);
            }
        } else {
            xmlhttp.onreadystatechange = function () {
                if (xmlhttp.readyState != 4) {
                    return;
                } else {
                    log("Async Response = " + xmlhttp.status + ": " + xmlhttp.responseText);
                    if (xmlhttp.status >= 200 && xmlhttp.status < 300 ) {
                        var responseObject = {
                            xmlDoc: xmlhttp.responseXML,
                            strHeaders: xmlhttp.getAllResponseHeaders(),
                            callback: callback,
                            requestType : reqType
                        };
                        objActions.onResponseComplete(responseObject);
                    } else {
                        objActions.onResponseComplete(this.unpackXMLErrorRsp(xmlhttp, reqType, callback));
                    }
                }
            }
        }
    },

    makeS3HTTPRequest : function (requestType, fileName, url, callback) {
        var curTime = new Date();
        curTime = curTime.toUTCString();

        if (requestType != "GET" &&
            requestType != "PUT" &&
            requestType != "HEAD") {
            return null;
        }

        var sigValues = new Array();
        sigValues.push(new Array("Request-Type", requestType));
        sigValues.push(new Array("Content-MD5", ""));
        sigValues.push(new Array("Content-Type", "text/plain"));
        sigValues.push(new Array("Date", curTime));
        sigValues.push(new Array("path", fileName));

        // Construct the string to sign and query string
        var strSig = "";
        var queryParams = "?";
        for (var i = 0; i < (sigValues.length - 1); ++i) {
            strSig += sigValues[i][1] + "\n";
        }

        // Append the last part of the request that needs to be signed.
        strSig += sigValues[i][1];

        log("StrSig ["+strSig+"]");

        var sig = "AWS " + this.accessCode + ":" + b64_hmac_sha1(this.secretKey, strSig);

        var xmlhttp = this.newInstance();
        xmlhttp.open(requestType, url, false);
        xmlhttp.setRequestHeader("Content-Length", "0");
        xmlhttp.setRequestHeader("Content-Type", "text/plain");
        xmlhttp.setRequestHeader("Content-MD5", "");
        xmlhttp.setRequestHeader("Date", curTime);
        xmlhttp.setRequestHeader("Authorization", sig);
        xmlhttp.setRequestHeader("User-Agent", this.USER_AGENT);

        log("Auth Str["+sig+"]");

        var timerKey = new Date().getTime();
        this.startTimer(timerKey, ec2ui_prefs.getRequestTimeout(), xmlhttp.abort);
        try {
            xmlhttp.send(null);
        } catch(e) {
            if (!this.stopTimer(timerKey)) {
                // A timer didn't exist, this is unexpected
                throw e;
            }

            var responseObject = {
                callback: callback,
                faultCode : "Timeout",
                faultString : "Your request timed out, you may want to have another stab at it",
                hasErrors : true
            };
            return responseObject;
        }

        log("Sync Response = " + xmlhttp.status + "("+xmlhttp.readyState+"): " + xmlhttp.responseText);

        this.stopTimer(timerKey);

        var fError = true;
        if (xmlhttp.status >= 200 && xmlhttp.status < 300 ) {
            fError = false;
        }

        var responseObject = {
            xmlhttp : xmlhttp,
            callback : callback,
            hasErrors : fError
        };

        return responseObject;

    },

    unpackXMLErrorRsp : function(xmlhttp, reqType, callback) {
        var faultCode = "Unknown";
        var faultString = "An unknown error occurred.";
        var requestId = "";
        var xmlDoc = xmlhttp.responseXML;

        var node = xmlDoc.getElementsByTagName("Code")[0].firstChild;
        if (node != null) {
            faultCode = node.nodeValue;
        }
        node = xmlhttp.responseXML.getElementsByTagName("Message")[0].firstChild;
        if (node != null) {
            faultString = node.nodeValue;
        }

        node = xmlhttp.responseXML.getElementsByTagName("RequestID")[0].firstChild;
        if (node != null) {
            requestId = node.nodeValue;
        }

        var responseObject = {
            xmlDoc: xmlDoc,
            strHeaders: xmlhttp.getAllResponseHeaders(),
            callback: callback,
            requestType : reqType,
            faultCode : faultCode,
            requestId : requestId,
            faultString : faultString,
            hasErrors : true
        };

        return responseObject;
    },

    queryCheckIP : function(reqType, retVal) {
        var xmlhttp = this.newInstance();
        xmlhttp.open("GET", "http://checkip.amazonaws.com/" + reqType, false);
        xmlhttp.setRequestHeader("User-Agent", this.USER_AGENT);

        var timerKey = new Date().getTime();
        this.startTimer(timerKey, 10000, xmlhttp.abort);
        try {
            xmlhttp.send(null);
            this.stopTimer(timerKey);
        } catch(e) {
            if (!this.stopTimer(timerKey)) {
                // A timer didn't exist, this is unexpected
                throw e;
            }
        }

        retVal.ipAddress = xmlhttp.responseText;
    },

    startTimer : function(key, timeout, expr) {
        var timer = window.setTimeout(expr, timeout);
        this.timers[key] = timer;
    },

    stopTimer : function(key, timeout) {
        var timer = this.timers[key];
        this.timers[key] = null;
        if (timer == null) {
            return false;
        }
        window.clearTimeout(timer);
        return true;
    }
}
