function encodeJSONMap(map) {
    var pairs = new Array();
    for (k in map) {
        if (map.hasOwnProperty(k)) {
            var v = map[k];
            if (v != null) {
                if (typeof v === 'object') {
                    pairs.push("'"+k+"':"+encodeJSONMap(v));
                } else if (typeof v != 'function') {
                    pairs.push("'"+k+"':'"+v+"'");
                }
            }
        }
    }
    return "({"+pairs.join(',')+"})";
}

function WrappedMapAccounts(map, prefs) {
    that = new WrappedMap(map);
    that.prefs = prefs;
    // Since this object is not yet formed, you need to pass
    // that to the prefs so that the map can be written out
    // to the preferences store.
    prefs.setAccountIdMap(that);

    that.put = function(key, value) {
        this.map[key] = value;
        this.prefs.setAccountIdMap(this);
    };

    that.removeKey = function(key) {
        this.map[key] = null;
        this.prefs.setAccountIdMap(this);
    };
    return that;
}

function WrappedMapEndpoints(map, prefs) {
    that = new WrappedMap(map);
    that.prefs = prefs;
    // Since this object is not yet formed, you need to pass
    // that to the prefs so that the map can be written out
    // to the preferences store.
    prefs.setEndpointMap(that);

    that.put = function(key, value) {
        this.map[key] = value;
        this.prefs.setEndpointMap(this);
    };

    that.removeKey = function(key) {
        this.map[key] = null;
        this.prefs.setEndpointMap(this);
    };
    return that;
}

function WrappedMap(map) {
    this.map = map;

    this.get = function(key) {
        return this.map[key];
    };

    this.put = function(key, value) {
        this.map[key] = value;
    };

    this.removeKey = function(key) {
        this.map[key] = null;
    };

    this.toArray = function(block) {
        var list = new Array();
        keys = this.keys();
        for (var i=0, len=keys.length; i<len; ++i) {
            v = this.map[keys[i]];
            list.push(block(keys[i], v));
        }
        return list;
    };

    this.keys = function() {
        var list = new Array();
        for (k in this.map) {
            if (this.map.hasOwnProperty(k) &&
                typeof this.map[k] != "function") {
                v = this.map[k];
                if (v != null) {
                    list.push(k);
                }
            }
        }
        return list;
    };

    this.toJSONString = function() {
        return encodeJSONMap(this.map);
    };
    return this;
};

var ec2ui_prefs = {
    ACTIVE_USER_NAME    : "ec2ui.active.credentials.name",
    ACTIVE_ENDPOINT     : "ec2ui.active.endpoint",
    RDP_COMMAND         : "ec2ui.tools.rdp.command",
    RDP_ARGS            : "ec2ui.tools.rdp.args",
    SSH_COMMAND         : "ec2ui.tools.ssh.command",
    SSH_ARGS            : "ec2ui.tools.ssh.args",
    SSH_KEY             : "ec2ui.tools.ssh.key",
    EC2_KEY_TEMPLATE    : "ec2ui.tools.ec2.key.template",
    SSH_USER            : "ec2ui.tools.ssh.user",
    DEBUG_ENABLED       : "ec2ui.debugging.enabled",
    OFFLINE             : "ec2ui.offline.enabled",
    QUERY_ON_START      : "ec2ui.queryonstart.enabled",
    REFRESH_ON_CHANGE   : "ec2ui.refreshonchange.enabled",
    REFRESH_BUNDLE_VIEW : "ec2ui.refreshBundleView.enabled",
    AUTOFETCH_LP        : "ec2ui.autofetchlaunchpermissions.enabled",
    OPEN_IN_NEW_TAB     : "ec2ui.usenewtab.enabled",
    EC2_URL             : "ec2ui.url",
    CURRENT_TAB         : "ec2ui.current.tab",
    REQUEST_TIMEOUT     : "ec2ui.timeout.request",
    KNOWN_ACCOUNT_IDS   : "ec2ui.known.account.ids",
    LAST_EC2_PKEY_FILE  : "ec2ui.last.ec2privatekey.file",
    ENDPOINTS           : "ec2ui.endpoints",
    endpoints           : null,

    prefs : null,

    init : function() {
        if (this.prefs == null) {
            this.prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);
            // Read and write back our preferences. This ensures that if they're not yet set
            // they're set to our defaults.
            this.setQueryOnStartEnabled(this.isQueryOnStartEnabled());
            this.setRefreshOnChangeEnabled(this.isRefreshOnChangeEnabled());
            this.setRefreshBundleViewEnabled(this.isRefreshBundleViewEnabled());
            this.setLastUsedAccount(this.getLastUsedAccount());
            this.setLastUsedEndpoint(this.getLastUsedEndpoint());
            this.setRDPCommand(this.getRDPCommand());
            this.setRDPArgs(this.getRDPArgs());
            this.setSSHCommand(this.getSSHCommand());
            this.setSSHArgs(this.getSSHArgs());
            this.setSSHKeyTemplate(this.getSSHKeyTemplate());
            this.setPrivateKeyTemplate(this.getPrivateKeyTemplate());
            this.setSSHUser(this.getSSHUser());
            this.setRequestTimeout(this.getRequestTimeout());
            this.setServiceURL(this.getServiceURL());
            this.setCurrentTab(this.getCurrentTab());
            this.setDebugEnabled(this.isDebugEnabled());
            this.setOfflineEnabled(this.isOfflineEnabled());
            this.setOpenInNewTabEnabled(this.isOpenInNewTabEnabled());
            this.setAutoFetchLaunchPermissionsEnabled(this.isAutoFetchLaunchPermissionsEnabled());
            this.setAccountIdMap(this.getAccountIdMap());
            this.setLastEC2PKeyFile(this.getLastEC2PKeyFile());
        }
    },

    setLastUsedAccount : function(value) { this.saveStringPreference(this.ACTIVE_USER_NAME, value); },
    setRDPCommand : function(value) { this.saveStringPreference(this.RDP_COMMAND, value); },
    setRDPArgs : function(value) { this.saveStringPreference(this.RDP_ARGS, value); },
    setSSHCommand : function(value) { this.saveStringPreference(this.SSH_COMMAND, value); },
    setSSHArgs : function(value) { this.saveStringPreference(this.SSH_ARGS, value); },
    setSSHKeyTemplate : function(value) { this.saveStringPreference(this.SSH_KEY, value); },
    setPrivateKeyTemplate : function(value) { this.saveStringPreference(this.EC2_KEY_TEMPLATE, value); },
    setSSHUser : function(value) { this.saveStringPreference(this.SSH_USER, value); },
    setRequestTimeout : function(value) { this.saveIntPreference(this.REQUEST_TIMEOUT, value); },
    setServiceURL : function(value) { this.saveStringPreference(this.EC2_URL, value); },
    setCurrentTab : function(value) { this.saveIntPreference(this.CURRENT_TAB, value); },
    setDebugEnabled : function(enabled) { this.saveBoolPreference(this.DEBUG_ENABLED, enabled); },
    setOfflineEnabled : function(enabled) { this.saveBoolPreference(this.OFFLINE, enabled); },
    setOpenInNewTabEnabled : function(enabled) { this.saveBoolPreference(this.OPEN_IN_NEW_TAB, enabled); },
    setQueryOnStartEnabled : function(enabled) { this.saveBoolPreference(this.QUERY_ON_START, enabled); },
    setRefreshOnChangeEnabled : function(enabled) { this.saveBoolPreference(this.REFRESH_ON_CHANGE, enabled); },
    setRefreshBundleViewEnabled : function(enabled) { this.saveBoolPreference(this.REFRESH_BUNDLE_VIEW, enabled); },
    setAutoFetchLaunchPermissionsEnabled : function(enabled) { this.saveBoolPreference(this.AUTOFETCH_LP, enabled); },
    setLastEC2PKeyFile : function(value) { this.saveEC2PKeyForUser(value, this.getLastUsedAccount()); },

    getLastUsedAccount : function() { return this.getStringPreference(this.ACTIVE_USER_NAME, ""); },
    getRDPCommand : function() { return this.getStringPreference(this.RDP_COMMAND, this.getDefaultRDPCommandArgs()[0]); },
    getRDPArgs : function() { return this.getStringPreference(this.RDP_ARGS, this.getDefaultRDPCommandArgs()[1]); },
    getSSHCommand : function() { return this.getStringPreference(this.SSH_COMMAND, this.getDefaultSSHCommandArgs()[0]); },
    getSSHArgs : function() { return this.getStringPreference(this.SSH_ARGS, this.getDefaultSSHCommandArgs()[1]); },
    getSSHKeyTemplate : function() { return this.getStringPreference(this.SSH_KEY, this.getDefaultSSHKeyTemplate()); },
    getPrivateKeyTemplate : function() { return this.getStringPreference(this.EC2_KEY_TEMPLATE, this.getDefaultPrivateKeyTemplate()); },
    getSSHUser : function() { return this.getStringPreference(this.SSH_USER, "root"); },
    getRequestTimeout : function() { return this.getIntPreference(this.REQUEST_TIMEOUT, 15000); },
    getServiceURL : function() { return this.getStringPreference(this.EC2_URL, "https://us-east-1.ec2.amazonaws.com"); },
    getCurrentTab : function() { return this.getIntPreference(this.CURRENT_TAB, 0); },
    getLastEC2PKeyFile : function() { return this.getEC2PKeyForUser(this.getLastUsedAccount()); },
    isDebugEnabled : function() { return this.getBoolPreference(this.DEBUG_ENABLED, false); },
    isOfflineEnabled : function() { return this.getBoolPreference(this.OFFLINE, false); },
    isOpenInNewTabEnabled : function() { return this.getBoolPreference(this.OPEN_IN_NEW_TAB, true); },
    isQueryOnStartEnabled : function() { return this.getBoolPreference(this.QUERY_ON_START, true); },
    isRefreshOnChangeEnabled : function() { return this.getBoolPreference(this.REFRESH_ON_CHANGE, true); },
    isRefreshBundleViewEnabled : function() { return this.getBoolPreference(this.REFRESH_BUNDLE_VIEW, true); },
    isAutoFetchLaunchPermissionsEnabled : function() { return this.getBoolPreference(this.AUTOFETCH_LP, false); },

    // default helpers

    // get a [cmd, argument-string] pair
    getDefaultRDPCommandArgs : function() {
        if (navigator.platform.match(/^Mac/)) {
            return ["/Applications/Remote Desktop Connection.app/Contents/MacOS/Remote Desktop Connection", ""];
        } else if (navigator.platform.match(/^Win/)) {
            return ["c:\\Windows\\System32\\mstsc.exe", '/v ${host}'];
        }

        // probably linux
        return ["/usr/bin/gnome-terminal", '-x /usr/bin/rdesktop -g 1440x900 -u administrator -p "${pass}" -x l ${host}'];
    },

    getEC2PKeyForUser : function (user) {
        var pref = this.LAST_EC2_PKEY_FILE + "." + user;
        return this.getStringPreference(pref, "");
    },

    saveEC2PKeyForUser : function (value, user) {
        var pref = this.LAST_EC2_PKEY_FILE + "." + user;
        return this.saveStringPreference(pref, value);
    },

    // get a [cmd, argument-string] pair
    getDefaultSSHCommandArgs : function() {
        if (navigator.platform.match(/^Mac/)) {
            // this will open a terminal in Mac OS X and run ssh
            //   applescript is chunky
            var cmdline = [
                'on run argv',
                '  tell app "System Events" to set termOn to (exists process "Terminal")',
                '  set cmd to "ssh -i " & item 1 of argv & " " & item 2 of argv',
                '  if (termOn) then',
                '    tell app "Terminal" to do script cmd',
                '  else',
                '    tell app "Terminal" to do script cmd in front window',
                '  end if',
                '  tell app "Terminal" to activate',
                'end run'];
            // turn into -e 'line1' -e 'line2' etc.
            cmdline = cmdline.map(function(s) { return "-e '" + s.replace(/^\s+/, '') + "'" }).join(" ");

            return ["/usr/bin/osascript", cmdline + " ${key} ${user}@${host}"];
        } else if (navigator.platform.match(/^Win/)) {
            // windows, guess where Putty is
            // wrap key in quotes, it may have spaces
            return ["c:\\Program Files\\Putty\\putty.exe", '-i "${key}" ${user}@${host}'];
        }

        // probably linux
        return ["/usr/bin/gnome-terminal", '-x /usr/bin/ssh -i "${key}" ${user}@${host}'];
    },

    getDefaultSSHKeyTemplate : function() {
        if (navigator.platform.match(/^Win/)) {
            return "${home}\\ec2-keys\\${keyname}.ppk";
        }

        return "${home}/ec2-keys/id_${keyname}";
    },

    getDefaultPrivateKeyTemplate : function() {
        if (navigator.platform.match(/^Win/)) {
            return "${home}\\ec2-keys\\${keyname}.pem";
        }

        return "${home}/ec2-keys/id_${keyname}";
    },

    // These ones manage a pseudo-complex pref. This preference is a JSON encoded
    // JavaScript "map" mapping account IDs to friendly names.
    setAccountIdMap : function(value) {
        this.saveStringPreference(this.KNOWN_ACCOUNT_IDS, value.toJSONString());
    },

    getAccountIdMap : function() {
        var packedMap = this.getStringPreference(this.KNOWN_ACCOUNT_IDS, null);
        if (packedMap == null) {
            return new WrappedMapAccounts({}, this);
        }
        // Unpack the map and return it
        var unpackedMap = eval(packedMap);

        return new WrappedMapAccounts(unpackedMap, this);
    },

    getDefaultEndpoints : function(session) {
        var me = this;
        var wrap = function (regionMap) {
            log ("Endpoints callback");
            me.endpoints = regionMap;
        }
        // session.client.setEndpoint(new Endpoint('eu-west-1', 'https://eu-west-1.ec2.amazonaws.com', '2008-12-01'));
        session.controller.describeRegions(wrap);
    },

    // These ones manage a pseudo-complex pref. This preference is a JSON
    // encoded JavaScript "map" mapping endpoints to friendly names.
    setEndpointMap : function(value) {
        this.saveStringPreference(this.ENDPOINTS, value.toJSONString());
    },

    getEndpointMap : function() {
        var packedMap = this.getStringPreference(this.ENDPOINTS, null);
        var endpointmap = null;

        if (packedMap != null && packedMap.length > 0) {
            // Unpack the map and return it
            var endpointmap = eval(packedMap);

            if (endpointmap['us-east-1'].version == null) {
                return new WrappedMapEndpoints(endpointmap, this);
            }
        }

        // Must overwrite the version style endpoints
        if (this.endpoints == null) {
            this.getDefaultEndpoints(ec2ui_session);
            log ("Using default endpoints");
        }

        // there is a chance that we retrieved the endpoints from the API.
        endpointmap = this.endpoints;

        // You couldn't retrieve the default endpoints. Hard code them
        if (endpointmap == null) {
            log ("Generating endpoints");
            endpointmap = new Object();
            endpointmap['us-east-1'] = new Endpoint('us-east-1', 'https://us-east-1.ec2.amazonaws.com');
            endpointmap['eu-west-1'] = new Endpoint('eu-west-1', 'https://eu-west-1.ec2.amazonaws.com');
        }

        return new WrappedMapEndpoints(endpointmap, this);
    },

    setLastUsedEndpoint : function(value) {
        // var strVal = value.toJSONString();
        this.saveStringPreference(this.ACTIVE_ENDPOINT, value);
    },

    getLastUsedEndpoint: function() {
        return this.getStringPreference(this.ACTIVE_ENDPOINT, "us-east-1");
    },

    getStringPreference : function(name, defValue) {
        if (this.prefs) {
            if (!this.prefs.prefHasUserValue(name)) {
                return defValue;
            }
            if (this.prefs.getPrefType(name) != this.prefs.PREF_STRING) {
                return defValue;
            }

            var prefValue = this.prefs.getCharPref(name).toString();
            if (prefValue.length == 0) {
                prefValue = defValue;
            }

            return prefValue;
        }

        return defValue;
    },

    getIntPreference : function(name, defValue) {
        if (this.prefs) {
            if (!this.prefs.prefHasUserValue(name)) {
                return defValue;
            }
            if (this.prefs.getPrefType(name) != this.prefs.PREF_INT) {
                return defValue;
            }

            return this.prefs.getIntPref(name);
        }

        return defValue;
    },

    getBoolPreference : function(name, defValue) {
        if (this.prefs) {
            if (!this.prefs.prefHasUserValue(name)) {
                return defValue;
            }
            if (this.prefs.getPrefType(name) != this.prefs.PREF_BOOL) {
                return defValue;
            }

            return this.prefs.getBoolPref(name);
        }

        return defValue;
    },

    saveStringPreference : function(name, value) {
        this.prefs.setCharPref(name, value);
    },

    saveIntPreference : function(name, value) {
        this.prefs.setIntPref(name, value);
    },

    saveBoolPreference : function(name, value) {
        this.prefs.setBoolPref(name, value);
    }
}
