//main object that holds the current session information
var ec2ui_session =
{
    accessCode      : "",
    secretKey       : "",
    initialized     : false,
    controller      : null,
    model           : null,
    credentials     : null,
    accountidmap    : null,
    endpointmap     : null,
    client          : null,

    initialize : function () {
        if (!this.initialized) {
            this.controller = ec2ui_controller;
            this.model = ec2ui_model;
            this.client = ec2_httpclient;
            ec2ui_prefs.init();

            document.getElementById("ec2ui.images.view").view = ec2ui_AMITreeView;
            document.getElementById("ec2ui.kernels.view").view = ec2ui_KernelTreeView;
            document.getElementById("ec2ui.ramdisks.view").view = ec2ui_RamdiskTreeView;
            document.getElementById("ec2ui.keypairs.view").view = ec2ui_KeypairTreeView;
            document.getElementById("ec2ui.instances.view").view = ec2ui_InstancesTreeView;
            document.getElementById("ec2ui.securitygroups.view").view = ec2ui_SecurityGroupsTreeView;
            document.getElementById("ec2ui.permissions.view").view = ec2ui_PermissionsTreeView;
            document.getElementById("ec2ui.eip.view").view = ec2ui_ElasticIPTreeView;
            document.getElementById("ec2ui.azones.view").view = ec2ui_AvailZoneTreeView;
            document.getElementById("ec2ui.volumes.view").view = ec2ui_VolumeTreeView;
            document.getElementById("ec2ui.snapshots.view").view = ec2ui_SnapshotTreeView;
            document.getElementById("ec2ui.bundleTasks.view").view = ec2ui_BundleTasksTreeView;

            if("@maone.net/noscript-service;1" in Components.classes) {
                (Components.classes ["@maone.net/noscript-service;1"].getService().wrappedJSObject).setJSEnabled("about:blank", true);
            }

            this.loadAccountIdMap();
            this.loadCredentials();
            this.switchCredentials();

            this.initialized = true;
        }

        this.loadEndpointMap();
        this.switchEndpoints();
    },

    tabSelectionChanged : function (event) {
        if (!this.initialized) { return; }
        var tabs = document.getElementById("ec2ui.tabs");

        var toCall = "invalidate()";
        if (this.getActiveCredential() != null &&
            ec2ui_prefs.isRefreshOnChangeEnabled()) {
            toCall = "refresh()";
        }

        switch (tabs.selectedItem.label) {
        case 'AMIs and Instances':
            eval("ec2ui_InstancesTreeView." + toCall);
            ec2ui_AMITreeView.invalidate();

            // Since the user can attach an EIP and an EBS volume to an
            // instance, we should refresh the respective lists of those
            // ec2 resources as well.
            // eval("ec2ui_VolumeTreeView." + toCall);
            // eval("ec2ui_ElasticIPTreeView." + toCall);

            break;
        case "Kernels and Ramdisks":
            ec2ui_KernelTreeView.invalidate();
            ec2ui_RamdiskTreeView.invalidate();
            break;
        case "KeyPairs":
            eval("ec2ui_KeypairTreeView." + toCall);
            break;
        case "Security Groups":
            eval("ec2ui_SecurityGroupsTreeView." + toCall);
            break;
        case "Elastic IPs":
            eval("ec2ui_ElasticIPTreeView." + toCall);
            break;
        case "Volumes and Snapshots":
            eval("ec2ui_VolumeTreeView." + toCall);
            eval("ec2ui_SnapshotTreeView." + toCall);
            break;
        case "Bundle Tasks":
            eval("ec2ui_BundleTasksTreeView." + toCall);
            break;
        case "Availability Zones":
            eval("ec2ui_AvailZoneTreeView." + toCall);
            break;
        default:
            log ("This is an invalid tab: " + tabs.selectedItem.label);
            break;
        }

        ec2ui_prefs.setCurrentTab(tabs.selectedIndex);
    },

    loadCredentials : function () {
        var activeCredsMenu = document.getElementById("ec2ui.active.credentials.list");
        activeCredsMenu.removeAllItems();

        var lastUsedCred = ec2ui_prefs.getLastUsedAccount();

        this.credentials = ec2ui_credentialManager.loadCredentials();
        for(var i in this.credentials) {
            activeCredsMenu.insertItemAt(
                    i,
                    this.credentials[i].name,
                    this.credentials[i].name
                    );
            if (lastUsedCred == this.credentials[i].name) {
                activeCredsMenu.selectedIndex = i;
            }
        }
    },

    getActiveCredential : function () {
        var activeCredsMenu = document.getElementById("ec2ui.active.credentials.list");

        if (this.credentials != null && this.credentials.length > 0) {
            if (activeCredsMenu.selectedIndex == -1) {
                activeCredsMenu.selectedIndex = 0;
            }
            return this.credentials[activeCredsMenu.selectedIndex];
        } else {
           return null;
        }
    },

    switchCredentials : function () {
        var activeCred = this.getActiveCredential();

        if (activeCred != null) {
            ec2ui_prefs.setLastUsedAccount(activeCred.name);
            this.client.setCredentials(activeCred.accessKey, activeCred.secretKey);
        } else {
            // There are no credentials in the system.
            // Let's ask the user to enter them
            var promptService =
                Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
                .getService(Components.interfaces.nsIPromptService);
            var text = "Would you like to provide Elasticfox with your AWS Credentials?";

            // if the user says no, the return value will not be 0.
            // In this case, just fall out
            if (promptService.confirmEx(
                    window,
                    "AWS Credentials Needed",
                    text,
                    promptService.STD_YES_NO_BUTTONS |
                    promptService.BUTTON_POS_0_DEFAULT,
                    "",
                    "",
                    "",
                    null,
                    {})) {
                // Reset the credentials stored in the client
                this.client.setCredentials("", "");
            } else {
                this.manageCredentials();
            }
        }

        // Since we are switching creds, ensure that
        // all the views are redrawn
        this.model.invalidate();

        // Set the active tab to the last tab we were viewing
        document.getElementById("ec2ui.tabs").selectedIndex = ec2ui_prefs.getCurrentTab();

        // The current tab's view needs to either
        // be invalidated or refreshed
        this.tabSelectionChanged();
    },

    manageEndpoints : function () {
        window.openDialog("chrome://ec2ui/content/dialog_manage_endpoints.xul", null, "chrome,centerscreen,modal", this.endpointmap);
        this.loadEndpointMap();
    },

    loadEndpointMap : function() {
        this.endpointmap = ec2ui_prefs.getEndpointMap();
        var activeEndpointsMenu = document.getElementById("ec2ui.active.endpoints.list");
        activeEndpointsMenu.removeAllItems();

        var lastUsedEndpoint = ec2ui_prefs.getLastUsedEndpoint();
        var endpointlist = this.endpointmap.toArray(function(k,v){return new Endpoint(k, v.url)});

        for(var i in endpointlist) {
            activeEndpointsMenu.insertItemAt(
                    i,
                    endpointlist[i].name,
                    endpointlist[i].name
                    );
            if (lastUsedEndpoint == endpointlist[i].name) {
                activeEndpointsMenu.selectedIndex = i;
            }
        }
    },

    getActiveEndpoint : function () {
        var activeEndpointname = document.getElementById("ec2ui.active.endpoints.list").value;
        if (activeEndpointname == null || activeEndpointname.length == 0) {
            activeEndpointname = ec2ui_prefs.getLastUsedEndpoint();
        }
        if (this.endpointmap == null) {
            return new Endpoint(activeEndpointname, ec2ui_prefs.getServiceURL());
        } else {
            return this.endpointmap.get(activeEndpointname);
        }
    },

    switchEndpoints : function () {
        var activeEndpoint = this.getActiveEndpoint();

        if (activeEndpoint != null) {
            ec2ui_prefs.setLastUsedEndpoint(activeEndpoint.name);
            ec2ui_prefs.setServiceURL(activeEndpoint.url);
            this.client.setEndpoint(activeEndpoint);
        } else {
            // There are no endpoints in the system.
            // Let's ask the user to enter them
            var promptService =
                Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
                .getService(Components.interfaces.nsIPromptService);
            var text = "Would you like to provide Elasticfox with an EC2 Endpoint?";

            // if the user says no, the return value will not be 0.
            // in this case, just fall out.
            if (promptService.confirmEx(
                    window,
                    "EC2 Endpoint Needed",
                    text,
                    promptService.STD_YES_NO_BUTTONS |
                    promptService.BUTTON_POS_0_DEFAULT,
                    "",
                    "",
                    "",
                    null,
                    {})) {
                // Reset the endpoint stored in the client and prefs
                this.client.setEndpoint(new Endpoint());
                ec2ui_prefs.setServiceURL("");
            } else {
                this.manageEndpoints();
            }
        }

        // Since we are switching creds, ensure that
        // all the views are redrawn
        this.model.invalidate();

        // Set the active tab to the last tab we were viewing
        document.getElementById("ec2ui.tabs").selectedIndex = ec2ui_prefs.getCurrentTab();

        // The current tab's view needs to either
        // be invalidated or refreshed
        this.tabSelectionChanged();
    },

    manageCredentials : function () {
        window.openDialog("chrome://ec2ui/content/dialog_manage_credentials.xul", null, "chrome,centerscreen,modal");
        this.loadCredentials();
    },

    manageTools : function () {
        window.openDialog("chrome://ec2ui/content/dialog_manage_tools.xul", null, "chrome,centerscreen,modal");
    },

    loadAccountIdMap : function () {
        this.accountidmap = ec2ui_prefs.getAccountIdMap();
    },

    manageAccountIds : function () {
        window.openDialog("chrome://ec2ui/content/dialog_manage_accountids.xul", null, "chrome,centerscreen,modal", this.accountidmap);
        this.loadAccountIdMap();
    },

    lookupAccountId : function(id) {
        if (this.accountidmap == null) {
           return id;
        }
        if (this.accountidmap.get(id) == null) {
           return id;
        }
        return this.accountidmap.get(id);
    },

    displayAbout : function () {
        window.openDialog("chrome://ec2ui/content/dialog_about.xul", null, "chrome,centerscreen,modal", ec2ui_session);
    },

    openURL : function(url) {
        window.open(url, "_new");
    },

    getApiVersion : function() {
        return this.client.API_VERSION;
    },

    showBusyCursor : function(fShow) {
        if (fShow) {
            document.getElementById("ec2ui-window").setAttribute("wait-cursor", true);
        } else {
            document.getElementById("ec2ui-window").removeAttribute("wait-cursor");
        }
    },

    openMainWindow: function () {
        var url = "chrome://ec2ui/content/ec2ui_main_window.xul";
        ec2ui_prefs.init();
        if (ec2ui_prefs.isOpenInNewTabEnabled()) {
            getBrowser().selectedTab = getBrowser().addTab(url);
        } else {
            getBrowser().selectedBrowser.contentDocument.location = url;
        }
    }
};
