import java.io.BufferedReader;
import java.io.File;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.HashMap;
import java.util.Map;
import java.util.Vector;

import org.apache.commons.logging.Log;
import org.kchine.r.workbench.RGui;
import org.kchine.rpf.PoolUtils;
import org.mozilla.browser.MozillaConfig;
import org.mozilla.browser.common.Platform;
import org.mozilla.browser.common.XULRunnerFinder;

public class S3foxBridge {
	
	private RGui _rgui;

	private static final Log log = org.apache.commons.logging.LogFactory.getLog(S3foxBridge.class);
	public S3foxBridge(RGui rgui) throws Exception {
		_rgui=rgui;		
		
		
		
		try {

            File xulrunnerDir = MozillaConfig.getXULRunnerHome();
            if (xulrunnerDir != null) {
                if (!XULRunnerFinder.isXULRunnerDir(xulrunnerDir)) {
                    // this isn't a valid home directory
                    log.error("invalid current xulrunner location "+xulrunnerDir.getAbsolutePath()); //$NON-NLS-1$
                    log.info("Continuing to search for other xulrunners"); //$NON-NLS-1$
                    xulrunnerDir = null;
                }
            }

            if (xulrunnerDir == null) {
                xulrunnerDir = XULRunnerFinder.findXULRunner();
                MozillaConfig.setXULRunnerHome(xulrunnerDir);
            }


            log.info("Using xul runner dir: " + xulrunnerDir.getAbsolutePath()); //$NON-NLS-1$
            
          
            
            final File foundXulrunnerDir = xulrunnerDir;


            final File profDir;
            if (MozillaConfig.isInitializeWithProfile()) {
                if (MozillaConfig.getProfileDir()!=null) {
                    //specified from outside
                    profDir = MozillaConfig.getProfileDir();
                } else {
                    //create temp file, and delete it
                    File f = File.createTempFile("swing-mozilla", ""); //$NON-NLS-1$ //$NON-NLS-2$
                    f.delete();
                    //use the temp file as a name for the temporary profile dir
                    profDir = f;
                    profDir.mkdirs();
                    MozillaConfig.setProfileDir(profDir);
                }
            } else {
                profDir = null;
            }
            
            File nativeLibsDir = foundXulrunnerDir.getParentFile();
            if (Platform.platform==Platform.OSX &&
                !new File(nativeLibsDir, "libcocoautils.jnilib").exists()) //$NON-NLS-1$
            {
                log.error("Unable to resolve location of MozSwing native libraries"); //$NON-NLS-1$
                log.info("Continuing assuming java.library.path is set"); //$NON-NLS-1$
                nativeLibsDir = null;
            }
            final File foundNativeLibsDir = nativeLibsDir;
            
            System.out.println("xulrunnerDir:"+xulrunnerDir);
            System.out.println("profDir:"+profDir);
            System.out.println("nativeLibsDir:"+nativeLibsDir);
            
            
    		Runtime rt = Runtime.getRuntime();
    		
            String xulrunnerBinary=xulrunnerDir+"/xulrunner"+(PoolUtils.isMacOs()?"-bin":"");            
    		if (!PoolUtils.isWindowsOs()) {
    			Process chmodProc=rt.exec(new String[]{"chmod", "u+x" , xulrunnerBinary});
    			int chmodExitVal = chmodProc.waitFor();
    			if (chmodExitVal != 0) throw new Exception("chmod exit code : " + chmodExitVal);
    		}

    		
    		Vector<String> envVector = new Vector<String>();
			{
				Map<String, String> osenv = System.getenv();
				String OS_PATH=osenv.get("PATH");
				if (OS_PATH==null) OS_PATH=osenv.get("Path");
				if (OS_PATH==null) OS_PATH="";			
				Map<String, String> env = new HashMap<String, String>(osenv);
				env.put("Path", xulrunnerDir + System.getProperty("path.separator")+ OS_PATH);
				for (String k : env.keySet()) {
					envVector.add(k + "=" + env.get(k));
				}
			}
    		
			
    		final Process proc = rt.exec(new String[] { (PoolUtils.isWindowsOs() || PoolUtils.isMacOs() ? xulrunnerDir+"/" : "") + "xulrunner" + (PoolUtils.isMacOs()?"-bin":"") + (PoolUtils.isWindowsOs() ? ".exe": ""), rgui.getPluginsDir() + "/mozillabrowser/moz/xulapps/s3fox/application.ini"}, envVector.toArray(new String[0]), xulrunnerDir);

    		final Vector<String> xulrunnerPrint = new Vector<String>();
    		final Vector<String> errorPrint = new Vector<String>();

    		new Thread(new Runnable() {
    			public void run() {
    				try {
    					BufferedReader br = new BufferedReader(new InputStreamReader(proc.getErrorStream()));
    					String line = null;
    					while ((line = br.readLine()) != null) {
    						System.out.println(line);
    						errorPrint.add(line);
    					}
    				} catch (Exception e) {
    					e.printStackTrace();
    				}
    			}
    		}).start();
    		

    		new Thread(new Runnable() {
    			public void run() {
    				try {
    					BufferedReader br = new BufferedReader(new InputStreamReader(proc.getInputStream()));
    					String line = null;
    					while ((line = br.readLine()) != null) {
    						System.out.println(line);
    						xulrunnerPrint.add(line);
    					}
    				} catch (Exception e) {
    					e.printStackTrace();
    				}
    			}
    		}).start();

    		//int exitVal = proc.waitFor();
    		//if (exitVal != 0) throw new Exception("ec2-metadata exit code : " + exitVal);
            
            

        } catch (Exception e) {
        	e.printStackTrace();
		}
		
	}
		

}
